/*
 * @(#)String2Java.java				1.00
 * 
 * Copyright (C) Ramin Rakhamimov
 * ramin32@gmail.com
 * http://www.ramrak.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.ramrak.applications;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.FileNotFoundException;
import java.util.Scanner;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;

import org.apache.commons.lang.StringEscapeUtils;

/**
 * Converts a String or file into an format easily paste-able into java String's
 * i.e: 
 *   Hello World    --->  "Hello World\n" +
 *   FJA-US         --->  "FJA-US";
 * 
 * @version     1.00,   May 29, 2008
 * @author      Ramin Rakhamimov
 */

public class String2Java implements Runnable, ActionListener{
	private static final int TEXT_WIDTH = 30;
	private static final int TEXT_HEIGHT = 15;
	
	private final JFrame frame;
	
	private final JTextArea inputTextArea;
	private final JTextArea outputTextArea;
	
	private final JScrollPane inputScrollPane;
	private final JScrollPane outputScrollPane;
	
	private final JButton convertButton;
	
	private final JMenuBar menuBar;
	
	private final JMenu fileMenu;
	private final JMenu aboutMenu;
	
	private final JMenuItem exitMenuItem;
	private final JMenuItem aboutMenuItem;
	private final JMenuItem openMenuItem;
	private final JLabel lengthLabel;
	private final JSplitPane splitPane;
	
	
	public String2Java(){
		frame = new JFrame("StringForJava");
		
		inputTextArea = new JTextArea(TEXT_HEIGHT,TEXT_WIDTH);
		outputTextArea = new JTextArea(TEXT_HEIGHT,TEXT_WIDTH);
		
		inputScrollPane = new JScrollPane(inputTextArea);
		outputScrollPane = new JScrollPane(outputTextArea);
		
		splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,inputScrollPane,outputScrollPane);
		
		convertButton = new JButton("Convert!");
		
		menuBar = new JMenuBar();
		
		fileMenu = new JMenu("File");
		aboutMenu = new JMenu("About");
		
		openMenuItem = new JMenuItem("Open");
		exitMenuItem = new JMenuItem("Exit");
		
		aboutMenuItem = new JMenuItem("About");
		lengthLabel = new JLabel();
	}
	
	public void run(){
		//Install listeners
		openMenuItem.addActionListener(this);
		exitMenuItem.addActionListener(this);
		aboutMenuItem.addActionListener(this);
		convertButton.addActionListener(this);
		
		//Add Menu Items to Menu's
		fileMenu.add(openMenuItem);
		fileMenu.add(exitMenuItem);
		aboutMenu.add(aboutMenuItem);
		
		//Add Menu's to Menu Bar
		menuBar.add(fileMenu);
		menuBar.add(aboutMenu);
		
		//Add everything to the frame
		frame.setJMenuBar(menuBar);
		frame.add(lengthLabel,BorderLayout.NORTH);
		
		// Set up the split pane
		splitPane.setOneTouchExpandable(true);
		splitPane.setDividerLocation(150);

		frame.add(splitPane,BorderLayout.CENTER);
		frame.add(convertButton,BorderLayout.SOUTH);
		
		//Finalize frame
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.pack();
		frame.setVisible(true);
	}
	
	public void actionPerformed(ActionEvent e) {
		if(e.getSource() == convertButton){
			String input = inputTextArea.getText();
			String output = convertString(input);
			outputTextArea.setText(output);
			lengthLabel.setText("Length: " + input.length());
		}
		else if(e.getSource() == openMenuItem){
			loadFileIntoTextArea(frame,inputTextArea);
		}
		else if(e.getSource() == exitMenuItem){
			confirmQuit();
		}
		else if(e.getSource() == aboutMenuItem){
			JOptionPane.showMessageDialog(frame, "String2Java\n"+
												"Ramin Rakhamimov\n"+
												"ramin.rakhamimov@fja-us.com\n"+
												"FJA-US");
		}
	}
	
	
	/**
	 * Loads a file into the given textArea using a JFileChooser
	 * @param frame Containing frame
	 * @param textArea input field for file
	 */
	private void loadFileIntoTextArea(JFrame frame,JTextArea textArea) {
		JFileChooser fileChooser = new JFileChooser();
		int success = fileChooser.showOpenDialog(frame);
		if(success == JFileChooser.APPROVE_OPTION){
			//Get the entire file and place it into textArea
			Scanner scan;
			try {
				scan = new Scanner(fileChooser.getSelectedFile()).useDelimiter("\\Z");
				textArea.setText(scan.next());
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			}
			
		}
	}

	/**
	 * Shows a confirm dialog, kills the application on yes.
	 */
	private void confirmQuit(){
		int option = JOptionPane.showConfirmDialog(frame, "Are you sure you want to quit?");
		if(option == JOptionPane.YES_OPTION)
			System.exit(0);
	}
	
	/**
	 * 
	 * @param input String to convert
	 * @return A converted String with quotes surrounding each line.
	 */
	public static String convertString(String input){
		Scanner scanner = new Scanner(input).useDelimiter("\n");
		String output = "";
		while(scanner.hasNext()){
			output += "\"" + StringEscapeUtils.escapeJava(scanner.next());
			if(scanner.hasNext())
				output += "\\n\" + \n";
			else
				output += "\"";
			
		}
		return output +";";
	}
	
	
	//The main method
	public static void main(String[] args){
		SwingUtilities.invokeLater(new String2Java());
	}

	
}
